@@ -1,6 +1,6 @@
 /*
   QtCurve KWin window decoration
-  Copyright (C) 2007 - 2010 Craig Drummond <craig.p.drummond@googlemail.com>
+  Copyright (C) 2007 - 2010 Craig Drummond <craig.p.drummond@gmail.com>
 
   based on the window decoration "Plastik":
   Copyright (C) 2003-2005 Sandro Giessl <sandro@giessl.com>
@@ -24,7 +24,6 @@
   Boston, MA 02110-1301, USA.
  */
 #define DRAW_INTO_PIXMAPS
-
 #include <KDE/KLocale>
 #include <QBitmap>
 #include <QDateTime>
@@ -41,9 +40,7 @@
 #include <KDE/KColorUtils>
 #include <KDE/KWindowInfo>
 #include <KDE/KIconEffect>
-#ifdef DRAW_INTO_PIXMAPS
 #include <KDE/KWindowSystem>
-#endif
 #include <qdesktopwidget.h>
 #include "qtcurveclient.h"
 #include "qtcurvebutton.h"
@@ -57,6 +54,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include "../style/fixx11h.h"
+#include "../style/qtcurve.h"
 #include <QX11Info>
 
 #if KDE_IS_VERSION(4, 3, 85)
@@ -91,45 +89,102 @@ static inline int tabCloseIconSize(int titleHeight)
 
 #endif
 
-static QPainterPath createPath(const QRectF &r, double radius, bool botOnly=false)
+int getProperty(WId wId, const Atom &atom)
 {
-    double       dr(radius * 2);
-    QPainterPath path;
 
-    if(botOnly)
+    unsigned char *data;
+    int           dummy;
+    unsigned long num,
+                  dummy2;
+    int           rv(-1);
+
+    if (Success==XGetWindowProperty(QX11Info::display(), wId, atom, 0L, 1, False, XA_CARDINAL, &dummy2, &dummy, &num, &dummy2, &data) && num>0)
     {
-        path.moveTo(r.right(), r.top());
-        path.lineTo(r.left(), r.top());
+        unsigned short val=*((unsigned short*)data);
+
+        if(val<512)
+            rv=val;
+        XFree(data);
     }
+    //else
+    //    *data = NULL; // superflous?!?
+    return rv;
+}
+
+int getMenubarSizeProperty(WId wId)
+{
+    static const Atom constAtom = XInternAtom(QX11Info::display(), MENU_SIZE_ATOM, False);
+    return getProperty(wId, constAtom);
+}
+
+int getStatusbarSizeProperty(WId wId)
+{
+    static const Atom constAtom = XInternAtom(QX11Info::display(), STATUSBAR_ATOM, False);
+    return getProperty(wId, constAtom);
+}
+
+int getOpacityProperty(WId wId)
+{
+    static const Atom constAtom = XInternAtom(QX11Info::display(), OPACITY_ATOM, False);
+    int o=getProperty(wId, constAtom);
+    return o<0 || o>100 ? 100 : o;
+}
+
+void getBgndSettings(WId wId, EAppearance &app, QColor &col)
+{
+    static const Atom constAtom = XInternAtom(QX11Info::display(), BGND_ATOM, False);
+
+    unsigned char *data;
+    int           dummy;
+    unsigned long num,
+                  dummy2;
+
+    if (Success==XGetWindowProperty(QX11Info::display(), wId, constAtom, 0L, 1, False, XA_CARDINAL, &dummy2, &dummy, &num, &dummy2, &data) && num>0)
+    {
+        unsigned long val=*((unsigned long*)data);
+        app=(EAppearance)(val&0xFF);
+        col.setRgb((val&0xFF000000)>>24, (val&0x00FF0000)>>16, (val&0x0000FF00)>>8);
+
+        XFree(data);
+    }
+    //else
+    //    *data = NULL; // superflous?!?
+}
+
+static QPainterPath createPath(const QRectF &r, double radiusTop, double radiusBot)
+{
+    QPainterPath path;
+    double       diameterTop(radiusTop*2),
+                 diameterBot(radiusBot*2);
+
+    if (radiusBot>1.0)
+        path.moveTo(r.x()+r.width(), r.y()+r.height()-radiusBot);
     else
-    {
-        path.moveTo(r.right(), r.top() + radius);
-        path.arcTo(r.right() - dr, r.top(), dr, dr, 0.0, 90.0);
-        path.lineTo(r.left() + radius, r.top());
-        path.arcTo(r.left(), r.top(), dr, dr, 90.0, 90.0);
-    }
-    path.lineTo(r.left(), r.bottom() - radius);
-    path.arcTo(r.left(), r.bottom() - dr, dr, dr, 180.0, 90.0);
-    path.lineTo(r.right() - radius, r.bottom());
-    path.arcTo(r.right() - dr, r.bottom() - dr, dr, dr,  270.0, 90.0);
-    if(botOnly)
-        path.lineTo(r.right(), r.top());
+        path.moveTo(r.x()+r.width(), r.y()+r.height());
+
+    if (radiusTop>1.0)
+        path.arcTo(r.x()+r.width()-diameterTop, r.y(), diameterTop, diameterTop, 0, 90);
+    else
+        path.lineTo(r.x()+r.width(), r.y());
+
+    if (radiusTop>1.0)
+        path.arcTo(r.x(), r.y(), diameterTop, diameterTop, 90, 90);
+    else
+        path.lineTo(r.x(), r.y());
+
+    if (radiusBot>1.0)
+        path.arcTo(r.x(), r.y()+r.height()-diameterBot, diameterBot, diameterBot, 180, 90);
+    else
+        path.lineTo(r.x(), r.y()+r.height());
+
+    if (radiusBot>1.0)
+        path.arcTo(r.x()+r.width()-diameterBot, r.y()+r.height()-diameterBot, diameterBot, diameterBot, 270, 90);
+    else
+        path.lineTo(r.x()+r.width(), r.y()+r.height());
 
     return path;
 }
 
-#if KDE_IS_VERSION(4, 3, 0)
-static QPainterPath createPath(const QRect &r, bool fullRound, bool inner=false, bool botOnly=false)
-{
-    double radius((fullRound ? 6.0 : 2.0) - (inner ? 1.0 : 0.0));
-    int    adjust(botOnly ? 0 : 6);
-    QRect  fr(inner ? r.adjusted(1, 1, -1, -1) : r.adjusted(0, 1, 0, 0));
-    QRectF rf(fr.x(), fr.y()+adjust, fr.width(), fr.height() - adjust);
-
-    return createPath(rf, radius, botOnly);
-}
-#endif
-
 static void drawSunkenBevel(QPainter *p, const QRect &r, const QColor &bgnd, bool circular, int round)
 {
     double          radius=circular
@@ -139,7 +194,7 @@ static void drawSunkenBevel(QPainter *p, const QRect &r, const QColor &bgnd, boo
                                 : round>ROUND_SLIGHT
                                     ? 3.0
                                     : 2.0;
-    QPainterPath    path(createPath(QRectF(r), radius));
+    QPainterPath    path(createPath(QRectF(r), radius, radius));
     QLinearGradient g(r.topLeft(), r.bottomLeft());
     QColor          black(Qt::black),
                     white(Qt::white);
@@ -185,31 +240,21 @@ static void paintTabSeparator(QPainter *painter, const QRect &r)
 }
 #endif
 
-static void fillBackground(bool customBgnd, QPainter &painter, const QColor &col, const QRect &r, 
-                           const QPainterPath path=QPainterPath(), const QRegion &mask=QRegion())
+static void fillBackground(EAppearance app, QPainter &painter, const QColor &col, const QRect &fillRect, const QRect &widgetRect, const QPainterPath path)
 {
-    if(customBgnd)
+    if(!IS_FLAT_BGND(app) || !widgetRect.isEmpty())
     {
-        QStyleOption opt;
+        QtCurve::Style::BgndOption opt;
         opt.state|=QtC_StateKWin;
-        opt.rect=r;
+        opt.rect=fillRect;
+        opt.widgetRect=widgetRect;
         opt.palette.setColor(QPalette::Window, col);
-        if(!mask.isEmpty())
-        {
-            painter.save();
-            if(!path.isEmpty())
-            {
-                painter.setPen(col);
-                painter.drawPath(path);
-            }
-            painter.setClipRegion(mask, Qt::IntersectClip);
-        }
+        opt.app=app;
+        opt.path=path;
         Handler()->wStyle()->drawPrimitive(QtC_PE_DrawBackground, &opt, &painter, NULL);
-        if(!mask.isEmpty())
-            painter.restore();
     }
     else if(path.isEmpty())
-        painter.fillRect(r, col);
+        painter.fillRect(fillRect, col);
     else
         painter.fillPath(path, col);
 }
@@ -482,12 +527,13 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
     bool                 active(isActive()),
                          colorTitleOnly(windowBorder&WINDOW_BORDER_COLOR_TITLEBAR_ONLY),
                          roundBottom(Handler()->roundBottom()),
-                         outerBorder(Handler()->outerBorder()),
                          preview(isPreview()),
                          blend(!preview && Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_BlendMenuAndTitleBar, NULL, NULL)),
-                         customBgnd(colorTitleOnly && Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_CustomBgnd, 0L, 0L)),
                          menuColor(windowBorder&WINDOW_BORDER_USE_MENUBAR_COLOR_FOR_TITLEBAR),
-                         separator(active && windowBorder&WINDOW_BORDER_SEPARATOR);
+                         separator(active && windowBorder&WINDOW_BORDER_SEPARATOR),
+                         maximized(isMaximized());
+    QtCurveConfig::Shade outerBorder(Handler()->outerBorder()),
+                         innerBorder(Handler()->innerBorder());
     const int            border(Handler()->borderEdgeSize()),
                          titleHeight(layoutMetric(LM_TitleHeight)),
                          titleEdgeTop(layoutMetric(LM_TitleEdgeTop)),
@@ -498,12 +544,33 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
                          round=Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_Round, NULL, NULL),
                          buttonFlags=Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleBarButtons, NULL, NULL);
     int                  rectX, rectY, rectX2, rectY2, shadowSize(0),
-                         opacity(compositing ? Handler()->opacity(active) : 100);
+                         kwinOpacity(compositing ? Handler()->opacity(active) : 100),
+                         opacity(kwinOpacity);
+    EAppearance          bgndAppearance=APPEARANCE_FLAT;
+    QColor               windowCol(widget()->palette().color(QPalette::Window));
+
+    getBgndSettings(windowId(), bgndAppearance, windowCol);
+
+    QColor               col(KDecoration::options()->color(KDecoration::ColorTitleBar, active)),
+                         fillCol(colorTitleOnly ? windowCol : col);
+                         // APPEARANCE_RAISED is used to signal flat background, but have background image!
+                         // In which case we still have a custom background to draw.
+    bool                 customBgnd=APPEARANCE_FLAT!=bgndAppearance,
+                         customShadows=
+#if KDE_IS_VERSION(4, 3, 0)             
+                            Handler()->customShadows()
+#else
+                            false
+#endif
+                            ;
 
     painter.setClipRegion(e->region());
 
+    if(!preview && compositing && 100==opacity)
+        opacity=getOpacityProperty(windowId());
+    
 #if KDE_IS_VERSION(4, 3, 0)
-    if(Handler()->customShadows())
+    if(customShadows)
     {
         shadowSize=Handler()->shadowCache().shadowSize();
 
@@ -527,25 +594,38 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
     }
 #endif
 
-    r.getCoords(&rectX, &rectY, &rectX2, &rectY2);
+    int   side(layoutMetric(LM_BorderLeft)),
+          bot(layoutMetric(LM_BorderBottom));
+    QRect widgetRect(widget()->rect().adjusted(shadowSize+side, shadowSize+titleBarHeight, -(shadowSize+side), -(shadowSize+bot)));
 
-    QColor col(KDecoration::options()->color(KDecoration::ColorTitleBar, active)),
-           windowCol(widget()->palette().color(QPalette::Window)),
-           fillCol(colorTitleOnly ? windowCol : col);
+    r.getCoords(&rectX, &rectY, &rectX2, &rectY2);
 
     if(!preview && (blend||menuColor) && -1==itsMenuBarSize)
     {
         QString wc(windowClass());
-        if(wc==QLatin1String("Navigator Firefox") ||
-            wc==QLatin1String("Mail Thunderbird"))
+        if(wc==QLatin1String("W Navigator Firefox browser") ||
+           wc==QLatin1String("W Navigator Firefox view-source") ||
+           wc==QLatin1String("W Mail Thunderbird 3pane") ||
+           wc==QLatin1String("W Mail Thunderbird addressbook") ||
+           wc==QLatin1String("W Mail Thunderbird messageWindow") ||
+           wc==QLatin1String("D Calendar Thunderbird EventDialog") ||
+           wc==QLatin1String("W Msgcompose Thunderbird Msgcompose") ||
+           wc==QLatin1String("D Msgcompose Thunderbird Msgcompose")) 
             itsMenuBarSize=QFontMetrics(QApplication::font()).height()+8;
-        else if(wc.startsWith(QLatin1String("VCLSalFrame.DocumentWindow OpenOffice.org")) ||
-                wc.startsWith(QLatin1String("VCLSalFrame OpenOffice.org")) ||
-                wc==QLatin1String("soffice.bin Soffice.bin"))
-            itsMenuBarSize=QFontMetrics(QApplication::font()).height()+9;
+            
+        else if(
+#if 0 // Currently LibreOffice does not seem to pain menubar backgrounds for KDE - so disable check here...
+                wc.startsWith(QLatin1String("W VCLSalFrame libreoffice-")) ||
+                wc.startsWith(QLatin1String("W VCLSalFrame.DocumentWindow libreoffice-")) ||
+                //wc==QLatin1String("W soffice.bin Soffice.bin") ||
+#endif
+                wc.startsWith(QLatin1String("W VCLSalFrame.DocumentWindow OpenOffice.org")) ||
+                wc.startsWith(QLatin1String("W VCLSalFrame OpenOffice.org")) ||
+                wc==QLatin1String("W soffice.bin Soffice.bin"))
+            itsMenuBarSize=QFontMetrics(QApplication::font()).height()+7;
         else
         {
-            int val=getProperty();
+            int val=getMenubarSizeProperty(windowId());
             if(val>-1)
                 itsMenuBarSize=val;
         }
@@ -560,47 +640,90 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
         double alpha=opacity/100.0;
         col.setAlphaF(alpha);
         windowCol.setAlphaF(alpha);
-        if(!Handler()->opaqueBorder())
-            fillCol.setAlphaF(alpha);
-    }
-
-    if(isMaximized())
-        painter.setClipRect(r, Qt::IntersectClip);
-    else
-#if KDE_IS_VERSION(4, 3, 0)
-        if(!compositing && !preview)
-#endif
-        painter.setClipRegion(getMask(round, r), Qt::IntersectClip);
-
-    if(!compositing)
-        fillBackground(customBgnd, painter, fillCol, r);
-
-    painter.setRenderHint(QPainter::Antialiasing, true);
-    if(compositing)
-    {
-#if KDE_IS_VERSION(4, 3, 0)
-        if(roundBottom)
-            fillBackground(customBgnd, painter, fillCol, r.adjusted(0, titleBarHeight, 0, 0), 
-                           customBgnd
-                                ? outerBorder
-                                    ? QPainterPath()
-                                    : createPath(r.adjusted(0, titleBarHeight-1, -1, 0), round>ROUND_SLIGHT, outerBorder, true).translated(0.5, 0.5) 
-                                : createPath(r.adjusted(0, titleBarHeight-1, 0, 0), round>ROUND_SLIGHT, outerBorder, true),
-                           customBgnd ? getMask(round, r.adjusted(0, 0, 0, outerBorder ? 0 : 1)) : QRegion());
-        else
-#endif
-            fillBackground(customBgnd, painter, fillCol, r.adjusted(0, titleBarHeight, 0, 0));
+        fillCol.setAlphaF(alpha);
     }
 
     opt.init(widget());
+    opt.state=QStyle::State_Horizontal|QStyle::State_Enabled|QStyle::State_Raised|
+             (active ? QStyle::State_Active : QStyle::State_None)|QtC_StateKWin;
 
-    if(isMaximized())
+    int   fillTopAdjust(maximized ? Handler()->borderEdgeSize() : 0),
+          fillBotAdjust(maximized ? Handler()->borderSize(true) : 0),
+          fillSideAdjust(maximized ? Handler()->borderSize(false) : 0);
+    QRect fillRect(r.adjusted(-fillSideAdjust, -fillTopAdjust, fillSideAdjust, fillBotAdjust));
+    bool  clipRegion((outerBorder || round<=ROUND_SLIGHT) && !maximized);
+
+    painter.setRenderHint(QPainter::Antialiasing, true);
+    if(clipRegion)
+    {
+        painter.save();
+        painter.setClipRegion(getMask(round, r));
+    }
+    else if(maximized)
+        painter.setClipRect(r, Qt::IntersectClip);
+
+    //
+    // Fill titlebar and border backgrounds...
+    //
+    QPainterPath fillPath(maximized || round<=ROUND_SLIGHT
+                                ? QPainterPath() 
+                                : createPath(QRectF(fillRect), 
+                                             APPEARANCE_NONE==Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleBarApp,
+                                                                                               &opt, NULL) ? 6.0 : 8.0, 
+                                             roundBottom ? 6.0 : 1.0));
+
+
+    bool fillTitlebar(windowBorder&WINDOW_BORDER_FILL_TITLEBAR),
+         opaqueBorder(opacity<100 && Handler()->opaqueBorder());
+
+    if(opaqueBorder)
+    {
+        QColor fc(fillCol);
+        fc.setAlphaF(1.0);
+        if(!clipRegion)
+            painter.save();
+        painter.setClipRect(r.adjusted(0, titleBarHeight, 0, 0), Qt::IntersectClip);
+        fillBackground(bgndAppearance, painter, fc, fillRect, widgetRect, fillPath);
+        if(fillTitlebar)
+        {
+            painter.save();
+            painter.setClipRect(QRect(r.x(), r.y(), r.width(), titleBarHeight), Qt::IntersectClip);
+            fillBackground(bgndAppearance, painter, fillCol, fillRect, widgetRect, fillPath);
+            painter.restore();
+        }
+    }
+    else
+    {
+        if(!fillTitlebar)
+        {
+            if(!clipRegion)
+                painter.save();
+            if(!compositing)
+            {
+                // Not compositing, so need to fill with background colour...
+                if(fillPath.isEmpty())
+                    painter.fillRect(QRect(r.x(), r.y(), r.width(), titleBarHeight), windowCol);
+                else
+                    painter.fillPath(fillPath, windowCol);
+            }
+
+            painter.setClipRect(r.adjusted(0, titleBarHeight, 0, 0), Qt::IntersectClip);
+        }
+        fillBackground(bgndAppearance, painter, fillCol, fillRect, widgetRect, fillPath);
+    }
+
+    if(clipRegion || opaqueBorder || !fillTitlebar)
+        painter.restore();
+    //
+    // Titlebar and border backgrounds filled.
+    //
+        
+    if(maximized)
         r.adjust(-3, -border, 3, 0);
     opt.palette.setColor(QPalette::Button, col);
     opt.palette.setColor(QPalette::Window, windowCol);
     opt.rect=QRect(r.x(), r.y()+6, r.width(), r.height()-6);
-    opt.state=QStyle::State_Horizontal|QStyle::State_Enabled|QStyle::State_Raised|
-             (active ? QStyle::State_Active : QStyle::State_None)|QtC_StateKWin;
+
 
     if(!roundBottom)
         opt.state|=QtC_StateKWinNotFull;
@@ -610,11 +733,17 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
 
     if(outerBorder)
     {
-        if(opacity<100)
+#if KDE_IS_VERSION(4, 3, 0)
+        if(QtCurveConfig::SHADE_SHADOW==outerBorder && customShadows)
         {
-            painter.save();
-            painter.setClipRect(r.adjusted(0, titleBarHeight, 0, 0), Qt::IntersectClip);
+            opt.version=2+TBAR_BORDER_VERSION_HACK;
+            opt.palette.setColor(QPalette::Shadow, blendColors(Handler()->shadowCache().color(active), windowCol, active ? 0.75 : 0.4));
         }
+        else
+#endif
+            opt.version=(QtCurveConfig::SHADE_LIGHT==outerBorder ? 0 : 1)+TBAR_BORDER_VERSION_HACK;
+        painter.save();
+        painter.setClipRect(r.adjusted(0, titleBarHeight, 0, 0), Qt::IntersectClip);
 #ifdef DRAW_INTO_PIXMAPS
         if(!compositing && !preview)
         {
@@ -635,25 +764,41 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
         else
 #endif
             Handler()->wStyle()->drawPrimitive(QStyle::PE_FrameWindow, &opt, &painter, widget());
-        if(opacity<100)
-            painter.restore();
+        painter.restore();
     }
     else
         opt.state|=QtC_StateKWinNoBorder;
 
-    opt.palette.setColor(QPalette::Button, col);
+    if(kwinOpacity<100 && (blend||menuColor))
+    {
+        // Turn off opacity for titlebar...
+        QColor c(col);
+        c.setAlphaF(1.0);
+        opt.palette.setColor(QPalette::Button, c);
+        c=windowCol;
+        c.setAlphaF(1.0);
+        opt.palette.setColor(QPalette::Window, c);
+    }
+
     opt.rect=QRect(r.x(), r.y(), r.width(), titleBarHeight);
     opt.titleBarState=(active ? QStyle::State_Active : QStyle::State_None)|QtC_StateKWin;
     
     if(!preview && !isShade() && blend && -1!=itsMenuBarSize)
         opt.rect.adjust(0, 0, 0, itsMenuBarSize);
 
+    // Remove QtC_StateKWinNotFull settings - as its the same value as QtC_StateKWinFillBgnd, and it is not
+    // used in CC_TitleBar...
+    if(!roundBottom)
+        opt.state&=~QtC_StateKWinNotFull;
+
 #ifdef DRAW_INTO_PIXMAPS
-    if(!compositing && !preview)
+    if(!compositing && !preview && !customBgnd)
     {
         QPixmap  tPix(32, titleBarHeight+(!blend || itsMenuBarSize<0 ? 0 : itsMenuBarSize));
         QPainter tPainter(&tPix);
         tPainter.setRenderHint(QPainter::Antialiasing, true);
+        if(!customBgnd)
+            opt.state|=QtC_StateKWinFillBgnd;
         opt.rect=QRect(0, 0, tPix.width(), tPix.height());
         Handler()->wStyle()->drawComplexControl(QStyle::CC_TitleBar, &opt, &tPainter, widget());
         tPainter.end();
@@ -664,18 +809,22 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
     else
 #endif
         Handler()->wStyle()->drawComplexControl(QStyle::CC_TitleBar, &opt, &painter, widget());
-
     
-    if(outerBorder && Handler()->innerBorder())
+    if(outerBorder && innerBorder)
     {
         QStyleOptionFrame frameOpt;
-        int side(layoutMetric(LM_BorderLeft)),
-            bot(layoutMetric(LM_BorderBottom));
-
 
         frameOpt.palette=opt.palette;
-        frameOpt.rect=widget()->rect().adjusted(shadowSize+side, shadowSize+titleBarHeight, -(shadowSize+side), -(shadowSize+bot))
-                                      .adjusted(-1, -1, 1, 1);
+#if KDE_IS_VERSION(4, 3, 0)
+        if(QtCurveConfig::SHADE_SHADOW==innerBorder && customShadows)
+        {
+            frameOpt.version=2+TBAR_BORDER_VERSION_HACK;
+            frameOpt.palette.setColor(QPalette::Shadow, blendColors(Handler()->shadowCache().color(active), windowCol, active ? 0.75 : 0.4));
+        }
+        else
+#endif
+            frameOpt.version=(QtCurveConfig::SHADE_LIGHT==innerBorder ? 0 : 1)+TBAR_BORDER_VERSION_HACK;
+        frameOpt.rect=widgetRect.adjusted(-1, -1, 1, 1);
         frameOpt.state=(active ? QStyle::State_Active : QStyle::State_None)|QtC_StateKWin;
         frameOpt.lineWidth=frameOpt.midLineWidth=1;
         Handler()->wStyle()->drawPrimitive(QStyle::PE_Frame, &frameOpt, &painter, widget());
@@ -683,16 +832,20 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
 
     if(buttonFlags&TITLEBAR_BUTTON_SUNKEN_BACKGROUND)
     {
-        int hOffset=2,
-            vOffset=hOffset+(outerBorder ? 1 :0),
-            posAdjust=isMaximized() || outerBorder ? 2 : 0;
+        int  hOffset=2,
+             vOffset=hOffset+(outerBorder ? 1 :0),
+             posAdjust=maximized || outerBorder ? 2 : 0,
+             edgePad=Handler()->edgePad();
+        bool menuIcon=TITLEBAR_ICON_MENU_BUTTON==Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleBarIcon, 0L, 0L),
+             menuOnlyLeft=menuIcon && onlyMenuIcon(true),
+             menuOnlyRight=menuIcon && !menuOnlyLeft && onlyMenuIcon(false);
         
-        if(buttonsLeftWidth()>(titleBarHeight-2*hOffset))
-            drawSunkenBevel(&painter, QRect(r.left()+hOffset+posAdjust, r.top()+vOffset,
-                                            buttonsLeftWidth()-hOffset, titleBarHeight-2*vOffset), col, buttonFlags&TITLEBAR_BUTTON_ROUND, round);
-        if(buttonsRightWidth()>(titleBarHeight-2*hOffset))
-            drawSunkenBevel(&painter, QRect(r.right()-(buttonsRightWidth()+posAdjust), r.top()+vOffset,
-                                            buttonsRightWidth(), titleBarHeight-2*vOffset), col, buttonFlags&TITLEBAR_BUTTON_ROUND, round);
+        if(!menuOnlyLeft && buttonsLeftWidth()>(titleBarHeight-2*hOffset))
+            drawSunkenBevel(&painter, QRect(r.left()+hOffset+posAdjust+edgePad, r.top()+vOffset+edgePad,
+                                            buttonsLeftWidth()-hOffset, titleBarHeight-2*(vOffset+edgePad)), col, buttonFlags&TITLEBAR_BUTTON_ROUND, round);
+        if(!menuOnlyRight && buttonsRightWidth()>(titleBarHeight-2*hOffset))
+            drawSunkenBevel(&painter, QRect(r.right()-(buttonsRightWidth()+posAdjust+edgePad), r.top()+vOffset+edgePad,
+                                            buttonsRightWidth(), titleBarHeight-2*(vOffset+edgePad)), col, buttonFlags&TITLEBAR_BUTTON_ROUND, round);
     }
     
     bool showIcon=TITLEBAR_ICON_NEXT_TO_TITLE==Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleBarIcon,  0L, 0L);
@@ -785,9 +938,9 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
 
     if(toggleButtons)
     {
-        if(!itsToggleMenuBarButton && toggleButtons&0x01 && (Handler()->wasLastMenu(windowId()) || getProperty()>-1))
+        if(!itsToggleMenuBarButton && toggleButtons&0x01 && (Handler()->wasLastMenu(windowId()) || getMenubarSizeProperty(windowId())>-1))
             itsToggleMenuBarButton=createToggleButton(true);
-        if(!itsToggleStatusBarButton && toggleButtons&0x02 && (Handler()->wasLastStatus(windowId()) || getProperty(false)>-1))
+        if(!itsToggleStatusBarButton && toggleButtons&0x02 && (Handler()->wasLastStatus(windowId()) || getStatusbarSizeProperty(windowId())>-1))
             itsToggleStatusBarButton=createToggleButton(false);
 
     //     if(itsHover)
@@ -813,7 +966,7 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
                     }
 
                     int     offset=2,
-                            posAdjust=isMaximized() ? 2 : 0;
+                            posAdjust=maximized ? 2 : 0;
                     QRect   cr(onLeft
                                 ? r.left()+buttonsLeftWidth()+posAdjust+constTitlePad+2
                                 : r.right()-(buttonsRightWidth()+posAdjust+constTitlePad+2+
@@ -850,21 +1003,11 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
 
     if(separator)
     {
-        EEffect       effect((EEffect)(Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleBarEffect)));
-        QColor        color(KDecoration::options()->color(KDecoration::ColorFont, isActive())),
-                      bgnd(KDecoration::options()->color(KDecoration::ColorTitleBar, isActive()));
+        QColor        color(KDecoration::options()->color(KDecoration::ColorFont, isActive()));
         Qt::Alignment align((Qt::Alignment)Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleAlignment, 0L, 0L));
 
         r.adjust(16, titleBarHeight-1, -16, 0);
-        if(EFFECT_NONE!=effect)
-        {
-            drawFadedLine(&painter, r,
-                          blendColors(WINDOW_SHADOW_COLOR(effect), bgnd, WINDOW_TEXT_SHADOW_ALPHA(effect)), true,
-                          align&(Qt::AlignHCenter|Qt::AlignRight), align&(Qt::AlignHCenter|Qt::AlignLeft));
-            r.adjust(0, -1, 0, 0);
-//             if (!isActive() && DARK_WINDOW_TEXT(color))
-//                 color=blendColors(color, bgnd, ((255 * 180) >> 8)/256.0);
-        }
+        color.setAlphaF(0.5);
         drawFadedLine(&painter, r, color, true, align&(Qt::AlignHCenter|Qt::AlignRight), align&(Qt::AlignHCenter|Qt::AlignLeft));
     }
     
@@ -1095,11 +1238,11 @@ QRegion QtCurveClient::getMask(int round, const QRect &r) const
                 }
                 else
                 {
-                    mask += QRegion(x, y+5, 1, h-6);
+                    mask += QRegion(x, y+5, 1, h-5);
                     mask += QRegion(x+1, y+3, 1, h-3);
                     mask += QRegion(x+2, y+2, 1, h-2);
                     mask += QRegion(x+3, y+1, 2, h-1);
-                    mask += QRegion(x+w-1, y+5, 1, h-6);
+                    mask += QRegion(x+w-1, y+5, 1, h-5);
                     mask += QRegion(x+w-2, y+3, 1, h-3);
                     mask += QRegion(x+w-3, y+2, 1, h-2);
                     mask += QRegion(x+w-5, y+1, 2, h-1);
@@ -1112,6 +1255,14 @@ QRegion QtCurveClient::getMask(int round, const QRect &r) const
     return QRegion();
 }
 
+bool QtCurveClient::onlyMenuIcon(bool left) const
+{
+    return QLatin1String("M")==
+                (left
+                    ? (options()->customButtonPositions() ? options()->titleButtonsLeft() : defaultButtonsLeft())
+                    : (options()->customButtonPositions() ? options()->titleButtonsRight() : defaultButtonsRight()));
+}
+
 QRect QtCurveClient::captionRect() const
 {
     QRect     r(widget()->rect());
@@ -1495,7 +1646,7 @@ void QtCurveClient::deleteSizeGrip()
     }
 }
 
-void QtCurveClient::informAppOfTitlebarSizeChanged()
+void QtCurveClient::informAppOfBorderSizeChanges()
 {
     static const Atom constQtCTitleBarSize = XInternAtom(QX11Info::display(), TITLEBAR_SIZE_ATOM, False);
 
@@ -1551,16 +1702,24 @@ void QtCurveClient::sendToggleToApp(bool menubar)
     }
 }
 
-const QString & QtCurveClient::windowClass(bool normalWindowsOnly)
+const QString & QtCurveClient::windowClass()
 {
     if(itsWindowClass.isEmpty())
     {
-        KWindowInfo info(windowId(), normalWindowsOnly ? NET::WMWindowType : 0, NET::WM2WindowClass);
-
-        if(normalWindowsOnly && NET::Normal!=info.windowType(NET::AllTypesMask))
-            itsWindowClass="<>";
-        else
-            itsWindowClass=info.windowClassName()+' '+info.windowClassClass();
+        KWindowInfo info(windowId(), NET::WMWindowType, NET::WM2WindowClass|NET::WM2WindowRole);
+        
+        switch(info.windowType(NET::AllTypesMask))
+        {
+            case NET::Dialog:
+                itsWindowClass="D "+info.windowClassName()+' '+info.windowClassClass()+' '+info.windowRole();
+                break;
+            case NET::Normal:
+                itsWindowClass="W "+info.windowClassName()+' '+info.windowClassClass()+' '+info.windowRole();
+                break;
+            default:
+                itsWindowClass="<>";
+                break;
+        }
     }
 
     return itsWindowClass;
@@ -1616,29 +1775,4 @@ QtCurveToggleButton * QtCurveClient::createToggleButton(bool menubar)
     return button;
 }
 
-int QtCurveClient::getProperty(bool menubar)
-{
-    static const Atom constQtcMenuSize  = XInternAtom(QX11Info::display(), MENU_SIZE_ATOM, False);
-    static const Atom constQtcStatusBar = XInternAtom(QX11Info::display(), STATUSBAR_ATOM, False);
-
-    unsigned char *data;
-    int           dummy;
-    unsigned long num,
-                  dummy2;
-    int           rv(-1);
-
-    if (Success==XGetWindowProperty(QX11Info::display(), windowId(), menubar ? constQtcMenuSize : constQtcStatusBar, 0L, 1, False,
-                                    XA_CARDINAL, &dummy2, &dummy, &num, &dummy2, &data) && num>0)
-    {
-        unsigned short val=*((unsigned short*)data);
-
-        if(val<512)
-            rv=val;
-        XFree(data);
-    }
-    //else
-    //    *data = NULL; // superflous?!?
-    return rv;
-}
-
 }
