@@ -2,7 +2,7 @@
 #define __QTCURVE_H__
 
 /*
-  QtCurve (C) Craig Drummond, 2007 - 2010 craig.p.drummond@googlemail.com
+  QtCurve (C) Craig Drummond, 2007 - 2010 craig.p.drummond@gmail.com
 
   ----
 
@@ -62,37 +62,44 @@ class QAbstractScrollArea;
 namespace QtCurve
 {
     class WindowManager;
-}
+    class BlurHelper;
+    class ShortcutHandler;
 
-class QtCurveStyle : public QCommonStyle
+class Style : public QCommonStyle
 {
     Q_OBJECT
     Q_CLASSINFO("X-KDE-CustomElements", "true")
 
     public:
 
-#if QT_VERSION < 0x040500
-    enum Version
-    {
-        VER_UNKNOWN,
-        VER_4x,  // <=4.4
-        VER_45   // >=4.5
-    };
-#endif
-
     enum BackgroundType
     {
         BGND_WINDOW,
         BGND_DIALOG,
         BGND_MENU
     };
+
+    enum MenuItemType
+    {
+        MENU_POPUP,
+        MENU_BAR,
+        MENU_COMBO
+    };
     
     enum CustomElements
     {
-        CE_QtC_KCapacityBar = CE_CustomBase+0xFFFF00,
-        CE_QtC_Preview
+        CE_QtC_KCapacityBar = CE_CustomBase+0x00FFFF00,
+        CE_QtC_Preview,
+        CE_QtC_SetOptions
     };
 
+    enum PreviewType
+    {
+        PREVIEW_FALSE,
+        PREVIEW_MDI,
+        PREVIEW_WINDOW
+    };
+    
     class PreviewOption : public QStyleOption
     {
         public:
@@ -100,6 +107,15 @@ class QtCurveStyle : public QCommonStyle
         Options opts;
     };
 
+    class BgndOption : public QStyleOption
+    {
+        public:
+
+        EAppearance  app;
+        QPainterPath path;
+        QRect        widgetRect;
+    };
+
     enum Icon
     {
         ICN_MIN,
@@ -115,11 +131,15 @@ class QtCurveStyle : public QCommonStyle
     };
 
 #ifdef QTC_STYLE_SUPPORT
-    QtCurveStyle(const QString &name=QString());
+    Style(const QString &name=QString());
 #else
-    QtCurveStyle();
+    Style();
 #endif
-    ~QtCurveStyle();
+
+    ~Style();
+    
+    void init(bool initial);
+    void freeColors();
 
     Options & options() { return opts; }
 
@@ -133,7 +153,7 @@ class QtCurveStyle : public QCommonStyle
 #endif
     void polishScrollArea(QAbstractScrollArea *scrollArea, bool isKFilePlacesView=false) const;
 
-    void unpolish(QApplication *app) { BASE_STYLE::unpolish(app); }
+    void unpolish(QApplication *app);
     void unpolish(QWidget *widget);
     bool eventFilter(QObject *object, QEvent *event);
     void timerEvent(QTimerEvent *event);
@@ -183,12 +203,15 @@ class QtCurveStyle : public QCommonStyle
     void drawLightBevel(QPainter *p, const QRect &r, const QStyleOption *option, const QWidget *widget, int round, const QColor &fill,
                         const QColor *custom=0, bool doBorder=true, EWidget w=WIDGET_OTHER) const;
     void drawLightBevelReal(QPainter *p, const QRect &r, const QStyleOption *option, const QWidget *widget, int round, const QColor &fill,
-                            const QColor *custom, bool doBorder, EWidget w, bool useCache, ERound realRound) const;
-    void drawGlow(QPainter *p, const QRect &r, EWidget w) const;
+                            const QColor *custom, bool doBorder, EWidget w, bool useCache, ERound realRound, bool onToolbar) const;
+    void drawGlow(QPainter *p, const QRect &r, EWidget w, const QColor *cols=0L) const;
     void drawEtch(QPainter *p, const QRect &r,  const QWidget *widget, EWidget w, bool raised=false, int round=ROUNDED_ALL) const;
     void drawBgndRing(QPainter &painter, int x, int y, int size, int size2, bool isWindow) const;
     QPixmap drawStripes(const QColor &color, int opacity) const;
-    void drawBackground(QWidget *widget, BackgroundType type) const;
+    void drawBackground(QPainter *p, const QColor &bgnd, const QRect &r, int opacity, BackgroundType type, EAppearance app,
+                        const QPainterPath &path=QPainterPath()) const;
+    void drawBackgroundImage(QPainter *p, bool isWindow, const QRect &r) const;
+    void drawBackground(QPainter *p, const QWidget *widget, BackgroundType type) const;
     QPainterPath buildPath(const QRectF &r, EWidget w, int round, double radius) const;
     QPainterPath buildPath(const QRect &r, EWidget w, int round, double radius) const;
     void buildSplitPath(const QRect &r, int round, double radius, QPainterPath &tl, QPainterPath &br) const;
@@ -201,30 +224,31 @@ class QtCurveStyle : public QCommonStyle
                         const QColor &iconColor, const QColor *btnCols, const QColor *bgndCols) const;
     bool drawMdiButton(QPainter *painter, const QRect &r, bool hover, bool sunken, const QColor *cols) const;
     void drawMdiIcon(QPainter *painter, const QColor &color, const QColor &bgnd, const QRect &r,
-                     bool hover, bool sunken, Icon icon, bool stdSize, bool drewFrame) const;
+                     bool hover, bool sunken, Icon iclearcon, bool stdSize, bool drewFrame) const;
     void drawIcon(QPainter *painter, const QColor &color, const QRect &r, bool sunken, Icon icon, bool stdSize=true) const;
     void drawEntryField(QPainter *p, const QRect &rx,  const QWidget *widget, const QStyleOption *option, int round,
                         bool fill, bool doEtch, EWidget w=WIDGET_ENTRY) const;
-    void drawMenuItem(QPainter *p, const QRect &r, const QStyleOption *option, bool mbi, int round, const QColor *cols) const;
-    void drawProgress(QPainter *p, const QRect &r, const QStyleOption *option, int round, bool vertical=false, bool reverse=false) const;
+    void drawMenuItem(QPainter *p, const QRect &r, const QStyleOption *option, MenuItemType type, int round, const QColor *cols) const;
+    void drawProgress(QPainter *p, const QRect &r, const QStyleOption *option, bool vertical=false, bool reverse=false) const;
     void drawArrow(QPainter *p, const QRect &rx, PrimitiveElement pe, QColor col, bool small=false, bool kwin=false) const;
     void drawSbSliderHandle(QPainter *p, const QRect &r, const QStyleOption *option, bool slider=false) const;
     void drawSliderHandle(QPainter *p, const QRect &r, const QStyleOptionSlider *option) const;
     void drawSliderGroove(QPainter *p, const QRect &groove, const QRect &handle, const QStyleOptionSlider *slider, const QWidget *widget) const;
-    void drawMenuOrToolBarBackground(QPainter *p, const QRect &r, const QStyleOption *option, bool menu=true, bool horiz=true) const;
+    int  getOpacity(const QWidget *widget, QPainter *p) const;
+    void drawMenuOrToolBarBackground(const QWidget *widget, QPainter *p, const QRect &r, const QStyleOption *option, bool menu=true,
+                                     bool horiz=true) const;
     void drawHandleMarkers(QPainter *p, const QRect &r, const QStyleOption *option, bool tb, ELine handles) const;
     void fillTab(QPainter *p, const QRect &r, const QStyleOption *option, const QColor &fill, bool horiz, EWidget tab, bool tabOnly) const;
     void colorTab(QPainter *p, const QRect &r, bool horiz, EWidget tab, int round) const;
     void shadeColors(const QColor &base, QColor *vals) const;
     const QColor * buttonColors(const QStyleOption *option) const;
     QColor         titlebarIconColor(const QStyleOption *option) const;
-    const QColor * popupMenuCols(const QStyleOption *option) const;
-    const QColor & popupMenuCol(int shade=ORIGINAL_SHADE) const;
+    const QColor * popupMenuCols(const QStyleOption *option=0L) const;
     const QColor * checkRadioColors(const QStyleOption *option) const;
     const QColor * sliderColors(const QStyleOption *option) const;
     const QColor * backgroundColors(const QColor &col) const;
     const QColor * backgroundColors(const QStyleOption *option) const
-        { return backgroundColors(option->palette.background().color()); }
+        { return option ? backgroundColors(option->palette.background().color()) : itsBackgroundCols; }
     const QColor * highlightColors(const QColor &col) const;
     const QColor * highlightColors(const QStyleOption *option, bool useActive) const
         { return highlightColors(option->palette.brush(useActive ? QPalette::Active : QPalette::Current, QPalette::Highlight).color()); }
@@ -241,15 +265,12 @@ class QtCurveStyle : public QCommonStyle
     QColor         menuStripeCol() const;
     QPixmap *      getPixmap(const QColor col, EPixmap p, double shade=1.0) const;
     int            konqMenuBarSize(const QMenuBar *menu) const;
-#if QT_VERSION < 0x040500
-    Version        qtVersion() const;
-#endif
     const QColor & checkRadioCol(const QStyleOption *opt) const;
     QColor         shade(const QColor &a, double k) const;
     void           shade(const color &ca, color *cb, double k) const;
     QColor         getLowerEtchCol(const QWidget *widget) const;
-    QPalette::ColorRole getTextRole(const QWidget *w, const QPainter *p, QPalette::ColorRole def) const;
     int            getFrameRound(const QWidget *widget) const;
+    void           unregisterArgbWidget(QWidget *w);
 
     private Q_SLOTS:
 
@@ -259,11 +280,10 @@ class QtCurveStyle : public QCommonStyle
                                                Qt::Orientation orientation, const QStyleOption *option,
                                                const QWidget *widget) const;
     void           kdeGlobalSettingsChange(int type, int);
-    void           titlebarSizeChangedChange();
+    void           borderSizesChanged();
     void           toggleMenuBar(unsigned int xid);
     void           toggleStatusBar(unsigned int xid);
     void           compositingToggled();
-    void           updateBlurRegions();
 
     private:
 
@@ -279,7 +299,7 @@ class QtCurveStyle : public QCommonStyle
 #endif
 #ifdef Q_WS_X11
     bool           isWindowDragWidget(QObject *o);
-    void           emitMenuSize(QWidget *w, unsigned short size);
+    void           emitMenuSize(QWidget *w, unsigned short size, bool force=false);
     void           emitStatusBarState(QStatusBar *sb);
 #endif
 
@@ -291,6 +311,7 @@ class QtCurveStyle : public QCommonStyle
                                        itsMenubarCols[TOTAL_SHADES+1],
                                        itsFocusCols[TOTAL_SHADES+1],
                                        itsMouseOverCols[TOTAL_SHADES+1],
+                                       *itsPopupMenuCols,
                                        *itsSliderCols,
                                        *itsDefBtnCols,
                                        *itsComboBtnCols,
@@ -299,12 +320,12 @@ class QtCurveStyle : public QCommonStyle
                                        *itsOOMenuCols,
                                        *itsProgressCols,
                                        itsButtonCols[TOTAL_SHADES+1],
-                                       itsLighterPopupMenuBgndCol,
                                        itsCheckRadioCol;
     bool                               itsSaveMenuBarStatus,
                                        itsSaveStatusBarStatus,
                                        itsUsePixmapCache,
-                                       itsIsPreview;
+                                       itsInactiveChangeSelectionColor;
+    PreviewType                        itsIsPreview;
     mutable QColor                     *itsSidebarButtonsCols;
     mutable QColor                     *itsActiveMdiColors;
     mutable QColor                     *itsMdiColors;
@@ -318,11 +339,14 @@ class QtCurveStyle : public QCommonStyle
     mutable const QWidget              *itsSbWidget;
     mutable QLabel                     *itsClickedLabel;
     QSet<QProgressBar *>               itsProgressBars;
+    QSet<QWidget *>                    itsTransparentWidgets;
     int                                itsProgressBarAnimateTimer,
                                        itsAnimateStep;
     QTime                              itsTimer;
     mutable QMap<int, QColor *>        itsTitleBarButtonsCols;
+#ifdef QTC_ENABLE_PARENTLESS_DIALOG_FIX_SUPPORT
     mutable QMap<QWidget *, QWidget *> itsReparentedDialogs;
+#endif
     mutable QList<int>                 itsMdiButtons[2]; // 0=left, 1=right
     mutable int                        itsTitlebarHeight;
 
@@ -332,9 +356,6 @@ class QtCurveStyle : public QCommonStyle
 #ifdef Q_WS_X11
     QDBusInterface                     *itsDBus;
     QList<QWidget *>                   itsPendingBlurUpdates;
-#endif
-#if QT_VERSION < 0x040500
-    mutable Version                    itsQtVersion;
 #endif
     mutable QScrollBar                 *itsSViewSBar;
     mutable QMap<QWidget *, QSet<QWidget *> > itsSViewContainers;
@@ -342,7 +363,13 @@ class QtCurveStyle : public QCommonStyle
     KComponentData                     itsComponentData;
 #endif
     QtCurve::WindowManager             *itsWindowManager;
-    mutable bool                       itsCompositingActive;
+    QtCurve::BlurHelper                *itsBlurHelper;
+    QtCurve::ShortcutHandler           *itsShortcutHandler;
+#ifdef QTC_STYLE_SUPPORT
+    QString                            itsName;
+#endif
 };
 
+}
+
 #endif
